with Ada.Containers.Ordered_Maps,
     Ada.Strings.Unbounded,
     Ada.Task_Identification,
     Ada.Real_Time,
     Ada.Execution_Time,
     Minfr.Minfr_State.Hierarchical_State,
     Minfr.State_Activity;

use Ada.Containers,
    Ada.Strings.Unbounded,
    Ada.Task_Identification,
    Ada.Real_Time,
    Ada.Execution_Time,
    Minfr.Minfr_State,
    Minfr.State_Activity;

package minfr.Activity_Processor is

   type type_ActPro_Period is new Natural;
   type type_ActPro_ID is new Natural;
   type type_ActPro_Cycles is new Natural;

   type Activity_Processor (<>) is tagged limited private;
   type Ptr_Activity_Processor is access all Activity_Processor;

   not overriding
   function Constr_Activity_Processor return Ptr_Activity_Processor;
   not overriding
   function Constr_Activity_Processor (str : string) return Ptr_Activity_Processor;


   not overriding
   function Get_State (this : Activity_Processor) return string;
   not overriding
   function Get_Name (this : Activity_Processor) return string;
   not overriding
   Procedure Set_Name (this : in out Activity_Processor; str :string);
   not overriding
   function Get_Cycles (this : Activity_Processor) return type_ActPro_Cycles;
   not overriding
   Procedure Start (this : in out Activity_Processor; obj : Ptr_Activity_Processor);
   not overriding
   Procedure Suspend (this : in out Activity_Processor);
   not overriding
   Procedure Resume (this : in out Activity_Processor);
   not overriding
   Procedure Stop (this : in out Activity_Processor);
   not overriding
   Procedure Set_Period (this : in out Activity_Processor; Per :type_ActPro_Period);
   not overriding
   function Get_Period (this : Activity_Processor) return type_ActPro_Period;
   not overriding
   Procedure Add_Activity (this : in out Activity_Processor; Act : Ptr_State_Activity);
   not overriding
   Procedure Remove_Activity (this : in out Activity_Processor; Act_Id : type_ActPro_ID);
   not overriding
   Procedure Reset_Activities (this : in out Activity_Processor);
   not overriding
   function gcd (this : Activity_Processor; P ,Q : type_ActPro_Period) return type_ActPro_Period;



--  	virtual string toString(void);
--  	friend void * thread_fun(void *);
--  	static int gcd(int p, int q);
--  };
--  void * thread_fun(void *);
--  }
private

   Debug : Boolean := false;
   --Time_Activity : CPU_Time;

   package Tabla is new Ada.Containers.Ordered_Maps
     (Key_Type => type_ActPro_ID,
      Element_Type => Ptr_State_Activity,
      "<" => "<");
   use  tabla;

   protected type Protec_Activity is
      ------------------protec publi-----------------------------
      not overriding
      function Protec_Get_Name return string;
      not overriding
      Procedure Protec_Set_Name (str :string);
      not overriding
      function Protec_Get_Cycles return type_ActPro_Cycles;
      not overriding
      Procedure Protec_Set_Period (Per :type_ActPro_Period);
      not overriding
      function Protec_Get_Period return type_ActPro_Period;
      not overriding
      Procedure Protec_Add_Activity (Act : Ptr_State_Activity);
      not overriding
      Procedure Protec_Remove_Activity (Act_Id : type_ActPro_ID);
      not overriding
      Procedure Protec_Reset_Activities;
      not overriding
      function Protec_Calculate_Period return type_ActPro_Period;
      not overriding
      function Protec_gcd (P ,Q : type_ActPro_Period) return type_ActPro_Period;
      ------------------Protegidas-----------------------------
      not overriding
      function Get_Stopped return boolean;
      not overriding
      Procedure Set_Stop_ped (bool : boolean);
       not overriding
      function Get_Suspended return boolean;
      not overriding
      Procedure Set_Suspended (bool : boolean);
      not overriding
      Procedure Set_Cycles (Cycle : type_ActPro_Cycles);
      not overriding
      function Get_Started return boolean;
      not overriding
      Procedure Set_Started (bool : boolean);

      ---------------------------------------------------------
      not overriding
      function Begin_My_Activities return tabla.Cursor;
      not overriding
      function Last_My_Activities return tabla.Cursor;
      not overriding
      function Element_My_Activities (iter : tabla.Cursor ) return Ptr_State_Activity;
      not overriding
      Procedure Length_My_Activities;
      not overriding
      Procedure Print_Table_My_Activities;

   private
      T: type_ActPro_Period;
      My_Activities : Tabla.Map;
      Star_ted : boolean;
      Stop_ped : boolean;
      Suspended : boolean;
      Cycles : type_ActPro_Cycles;
      Name : Unbounded_string :=Null_Unbounded_String; --Ristra de longitud Dinamica
   end Protec_Activity;

   task type Worker (actp : Ptr_Activity_Processor);
   type Ptr_Worker is access Worker;



   type Activity_Processor is tagged limited record
      Actvity_Data_Protec : Protec_Activity;
      Tarea :Ptr_Worker;
      My_Thread_ID : Task_Id := Null_Task_Id;
   end record;


   not overriding
   function Calculate_Period (this : Activity_Processor) return type_ActPro_Period;
   not overriding
   Procedure Chage_State (this : Activity_Processor; Pt_Bool : access boolean; bool : boolean);
   not overriding
   Procedure Run (this : in out Activity_Processor);



end minfr.Activity_Processor;
