with  Ada.Containers.Indefinite_Ordered_Maps,
     Ada.Containers.Doubly_Linked_Lists,
     Ada.Strings.Unbounded,
     Minfr.Msg_Aux,
     Minfr.Msg.Minfr_Msg,
     Ada.Text_IO,
     Ada.Integer_Text_IO,
     minfr.Minfr_State;

use Ada.Containers,
    Ada.Strings.Unbounded,
    Minfr.Msg_Aux,
    Minfr.Msg.Minfr_Msg,
    Ada.Text_IO,
    Ada.Integer_Text_IO,
    minfr.Minfr_State;

package minfr.MinFr_Component_Data.MinFr_Data is

   type Type_MD_State_Id is new Natural;

   type MinFr_Data (<>) is new MinFr_Component_Data with private;
   type P_MinFr_Data is access MinFr_Data;


    ---------Constructores----------
   not overriding
   function Constr_MinFr_Data (name : in string) return P_MinFr_Data;
   not overriding
   function Get_Name (this : in MinFr_Data) return string;

   overriding
   function Get_Complete_Name (this : in MinFr_Data;Name : in string; Type_Msg : Msg_Type;Subs_Id : LastTable_Id) return String;
   overriding
   function Get_Partial_Name (this : in MinFr_Data;Name : in string; Type_Msg : Msg_Type) return string;

   ----------------------------------------minfr_State---------------------------------------------
   not overriding
   procedure Register_State (this : in out MinFr_Data; State : Minfr_State_Ptr);
   not overriding
   function get_State (this : in MinFr_Data; St_Id : Type_MD_State_Id) return Minfr_State_Ptr;
   not overriding
   procedure Print_State (this : in MinFr_Data);

----------------------------------------OutPort_Msg---------------------------------------------
   overriding
   function Pop_OutPort_Msg (This :MinFr_Data;Name: String) return Minfr_Msg;
   overriding
   procedure Push_OutPort_Msg (This : MinFr_Data;Name: String;Msg :Ptr_Minfr_Msg);
   overriding
   function Has_OutPort_Msg (This : in MinFr_Data;Name : String) return Boolean;
   overriding
   procedure Create_OutPort_Buffer (This : in out MinFr_Data;Name : String;Bool : in out Boolean);

----------------------------------------Inport_Msg---------------------------------------------
   overriding
   procedure Subscribe_To_InPort_Msg (This : in out MinFr_Data;Name: String; Id : in out LastTable_Id);
   overriding
   procedure Push_In_Port_Msg (This : MinFr_Data;Name: String;Msg :Ptr_Minfr_Msg);
   overriding
   function Pop_InPort_Msg (This : MinFr_Data;Name: String;Subs_Id : LastTable_Id) return Minfr_Msg;
   overriding
   function Has_InPort_Msg (This : in MinFr_Data;Name : String;Subs_Id : LastTable_Id) return Boolean;
   overriding
   function Pop_UntilLast_InPort_Msg (This :MinFr_Data;Name: String;Subs_Id : LastTable_Id) return Minfr_Msg;

----------------------------------------Event_Msg-----------------------------------------------

   overriding
   procedure Subscribe_To_Event_Msg (this : in out MinFr_Data; Name : String; Id : in out LastTable_Id);
   overriding
   procedure Push_Event_Msg (This : MinFr_Data;Name: String;Msg :Ptr_Minfr_Msg);
   overriding
   function Pop_Event_Msg (This : MinFr_Data;Name: String;Subs_Id : LastTable_Id) return Minfr_Msg;
   overriding
   function Has_Event_Msg (This : in MinFr_Data;Name : String;Subs_Id : LastTable_Id) return Boolean;
   overriding
   function Pop_UntilLast_Event_Msg (This : MinFr_Data;Name: String;Subs_Id : LastTable_Id) return Minfr_Msg;

-------------------------------------------------------------------------------------------------

private


   Debug : boolean := False;
   --Msg1 : Minfr_Msg;
   --Msg2 : Minfr_Msg;
   --Msg_L : Minfr_Msg;
   --Msg_C : Minfr_Msg;
   --Msg_L : Ptr_Minfr_Msg := Constr_Minfr_Msg("msgL");
   --Msg_C : Ptr_Minfr_Msg :=Constr_Minfr_Msg("msgC");
   --type P_Minfr_Msg is access Minfr_Msg;

   package MsgQueue is new Ada.Containers.Doubly_Linked_Lists
     (Element_Type => Minfr_Msg,
      "=" => "=");
   use MsgQueue;


   type P_MsgQueue is access MsgQueue.List;

   package LastIdTable is new Ada.Containers.Indefinite_Ordered_Maps
     (Key_Type => string,
      Element_Type => LastTable_Id,
      "<" => "<");
   use  LastIdTable;



    package QueueTable is new Ada.Containers.Indefinite_Ordered_Maps
     (Key_Type => string,
      Element_Type => P_MsgQueue,
      "<" => "<");
   use QueueTable;

   package StateTable is new Ada.Containers.Indefinite_Ordered_Maps
     (Key_Type => State_Id,
      Element_Type => Minfr_State_Ptr,
      "<" => "<");
    use StateTable;


   type MinFr_Data is new  MinFr_Component_Data with record
      Queue_Table :QueueTable.Map;
      Last_Queue_Subs_Id : LastIdTable.Map;
      My_Data : Unbounded_String;
      State_Table : StateTable.Map;
   end record;


   not overriding
   procedure Subscribe_To_Minfr_Msg (This : in out MinFr_Data;Name : String; Type_Msg  :Msg_Type;Id : in out LastTable_Id);
   not overriding
   procedure Push_Minfr_Msg(This : MinFr_Data;Name :String; Msg :Ptr_Minfr_Msg; Type_Msg :Msg_Type);
   not overriding
   function Pop_Minfr_Msg (This : MinFr_Data;Name :String;SubsId : LastTable_Id;Type_Msg :Msg_Type) return Minfr_Msg;
   not overriding
   function Has_Minfr_Msg(This : MinFr_Data; Name :String; SubsId : LastTable_Id; Type_Msg :Msg_Type) return Boolean;
   not overriding
   function Pop_Until_Last (This : MinFr_Data; Name :String; SubsId : LastTable_Id; Type_Msg :Msg_Type) return Minfr_Msg;


end minfr.MinFr_Component_Data.MinFr_Data;
